<?php

use Acms\Services\Facades\Storage;
use Acms\Services\Facades\Image;
use Acms\Services\Facades\Config;

class ACMS_GET_Admin_CheckList extends ACMS_GET
{
    function get()
    {
        if ( !sessionWithSubscription() ) return '';

        $Tpl    = new Template($this->tpl, new ACMS_Corrector());
        $DB     = DB::singleton(dsn());
        $keyword = $this->Get->get('keyword');

        if ( !empty($keyword) ) {
            $SQL = SQL::newSelect('config', 'config');
            $SQL->addLeftJoin('config_set', 'config_set_id', 'config_set_id', 'config_set', 'config');
            $SQL->addLeftJoin('module', 'config_module_id', 'module_id');
            $SQL->addLeftJoin('rule', 'config_rule_id', 'rule_id');
            $SQL->addWhereOpr('config_key', '%'.$keyword.'%', 'LIKE', 'OR');
            $SQL->addWhereOpr('config_value', '%'.$keyword.'%', 'LIKE', 'OR');
            $SQL->setLimit(300);
            $configAll = $DB->query($SQL->get(dsn()), 'all');

            if (is_array($configAll) && count($configAll) > 0) {
                foreach ( $configAll as $config ) {
                    $bid = $config['config_blog_id'];
                    $setId = $config['config_set_id'];
                    $rid = $config['config_rule_id'];
                    $mid = $config['config_module_id'];
                    $configVars = array(
                        'bid'   => $bid,
                        'setId' => $setId,
                        'rid'   => $rid,
                        'mid'   => $mid,
                        'configSetName' => $config['config_set_name'],
                        'rcode' => $config['rule_name'],
                        'mcode' => $config['module_identifier'],
                        'key'   => $config['config_key'],
                        'value' => $config['config_value'],
                    );
                    $configVars['blogUrl'] = acmsLink(array(
                        'bid' => $bid,
                        'admin' => 'config_index',
                    ));
                    $configVars['configSetUrl'] = acmsLink(array(
                        'bid' => $bid,
                        'query' => array(
                            'setid' => $setId,
                        ),
                        'admin' => 'config_index',
                    ));
                    $configVars['ruleUrl'] = acmsLink(array(
                        'bid' => $bid,
                        'query' => array(
                            'setid' => $setId,
                            'rid' => $rid,
                        ),
                        'admin' => 'config_index',
                    ));
                    $configVars['moduleUrl']  = acmsLink(array(
                        'bid' => $bid,
                        'query' => array(
                            'rid' => $rid,
                            'mid' => $mid,
                        ),
                        'admin' => 'module_edit',
                    ));
                    $Tpl->add(array('config:loop', 'config'), $configVars);
                }
                $Tpl->add('config');
            }
        }

        if ( LICENSE_BLOG_LIMIT == 2147483647 ) {
            $Tpl->add(array('userUnlimited', 'license'));
        } else {
            $Tpl->add(array('userLimited', 'license'), array(
                'limit' => LICENSE_BLOG_LIMIT,
            ));
        }

        //-------------
        // debug mode
        $debugMode['mode'] = isDebugMode() ? 'ON' : 'OFF';
        if (isDebugMode()) {
            $debugMode['caution'] = 'caution';
        }
        $Tpl->add('debugMode', $debugMode);

        //-------------
        // debug mode
        $benchmarkMode['mode'] = isBenchMarkMode() ? 'ON' : 'OFF';
        if (isBenchMarkMode()) {
            $benchmarkMode['caution'] = 'caution';
        }
        $Tpl->add('benchmarkMode', $benchmarkMode);

        //------------
        // 画像エンジン
        if ( class_exists('Imagick') && config('image_magick') == 'on' ) {
            $Tpl->add('imgLibrary', array(
                'mode'  => 'ImageMagick',
            ));
        } else {
            $Tpl->add('imgLibrary', array(
                'mode'  => 'GD',
            ));
        }

        //------------
        // ロスレス圧縮
        $Tpl->add('imgOptimizer', array(
            'format' => implode(', ', $this->imgOptimizerCheck()),
        ));

        //-------
        // cache
        $SQL = SQL::newSelect('blog');
        $SQL->setOrder('blog_id');

        foreach ( $DB->query($SQL->get(dsn()), 'all') as $blog ) {
            $bid = $blog['blog_id'];
            $this->addBlogInfo($Tpl, $bid);

            $SQL = SQL::newSelect('rule');
            $SQL->addSelect('rule_id');
            $SQL->addWhereOpr('rule_blog_id', $bid);
            foreach ( $DB->query($SQL->get(dsn()), 'all') as $rule ) {
                $rid = $rule['rule_id'];
                $this->addBlogInfo($Tpl, $bid, $rid);
            }
        }

        //------
        // form
        $SQL = SQL::newSelect('form');
        $SQL->addOrder('form_blog_id');
        $SQL->addOrder('form_id');
        $formAll = $DB->query($SQL->get(dsn()), 'all');

        if ( is_array($formAll) ) {
            foreach ( $formAll as $form ) {
                $formField = unserialize($form['form_data']);
                $formVars = $this->buildField($formField, $Tpl, array('formGeneral:loop'));
                $formVars['bid']    = $form['form_blog_id'];
                $formVars['fmid']   = $form['form_id'];
                $formVars['editUrl']  = acmsLink(array(
                    'bid'       => $form['form_blog_id'],
                    'query'     => array(
                        'fmid'   => $form['form_id'],
                    ),
                    'admin'     => 'form_edit',
                ));

                $Tpl->add('formGeneral:loop', $formVars);

                $formVars = $this->buildField($formField, $Tpl, array('formAdmin:loop'));
                $formVars['bid']    = $form['form_blog_id'];
                $formVars['fmid']   = $form['form_id'];
                $formVars['editUrl']  = acmsLink(array(
                    'bid'       => $form['form_blog_id'],
                    'query'     => array(
                        'fmid'   => $form['form_id'],
                    ),
                    'admin'     => 'form_edit',
                ));
                $Tpl->add('formAdmin:loop', $formVars);
            }
        }
        if ( !empty($keyword) ) {
            $Tpl->add(null, array(
                'keyword'   => $keyword,
            ));
        }

        return $Tpl->get();
    }

    function addBlogInfo(& $Tpl, $bid=0, $rid=null)
    {
        $configSetId = null;
        if ($setId = ACMS_RAM::blogConfigSetId($bid)) {
            $configSetId = $setId;
        }
        if ($configSetId === null) {
            if ($setId = Config::getAncestorBlogConfigSet($bid, 'config')) {
                $configSetId = $setId;
            }
        }

        $config = new Field;
        $config->overload(Config::loadDefaultField());
        $config->overload(Config::loadBlogConfigSet($bid));

        if ($rid && $rid > 0) {
            if ($ruleConfigSet = Config::loadRuleConfigSet($rid)) {
                $config->overload($ruleConfigSet);
            }
        }

        $blogConfig = array(
            'bid' => $bid,
            'blogName' => ACMS_RAM::blogName($bid),
            'setid' => $configSetId,
            'setName' => ACMS_RAM::configSetName($configSetId),
            'rid' => $rid,
            'ruleName' => ACMS_RAM::ruleName($rid),
            'cache' => $config->get('cache'),
            'cacheCaution' => $config->get('cache') === 'off' ? 'caution' : '',
            'cacheClearWhenPost' => $config->get('cache_clear_when_post'),
            'subscriberCache' => $config->get('subscriber_cache'),
            'cacheClearTarget' => $config->get('cache_clear_target'),
            'clientCache' => $config->get('cache_expire_client'),

        );
        $blogConfig['editUrl'] = acmsLink(array(
            'bid' => $bid,
            'query' => array(
                'setid' => $setId,
                'rid' => $rid,
            ),
            'admin' => 'config_cache',
        ));
        $Tpl->add('blog:loop', $blogConfig);
    }

    function imgOptimizerCheck()
    {
        $format = array();
        if ( 0
            || !Storage::isWritable(THEMES_DIR . 'system/images/system/check.jpeg')
            || !Storage::isWritable(THEMES_DIR . 'system/images/system/check.png')
            || !Storage::isWritable(THEMES_DIR . 'system/images/system/check.gif')
        ) {
            $format[] = 'Permission denied';
            return  $format;
        }
        if (Image::optimizeTest(THEMES_DIR . 'system/images/system/check.jpeg')) {
            $format[] = 'jpeg';
        }
        if (Image::optimizeTest(THEMES_DIR . 'system/images/system/check.png')) {
            $format[] = 'png';
        }
        if (Image::optimizeTest(THEMES_DIR . 'system/images/system/check.gif')) {
            $format[] = 'gif';
        }
        return $format;
    }

    function config($key=null, $bid=1, $rid=null)
    {
        $DB = DB::singleton(dsn());
        $SQL = SQL::newSelect('config');
        $SQL->addSelect('config_value');
        $SQL->addWhereOpr('config_blog_id', $bid);
        $SQL->addWhereOpr('config_key', $key);
        $SQL->addWhereOpr('config_rule_id', $rid);

        if ($config = $DB->query($SQL->get(dsn()), 'one')) {
            return $config;
        } else {
            $config = loadDefaultConfig();
            if (isset($config[$key])) {
                return $config[$key];
            }
        }
        return false;
    }
}
